-- -*- haskell -*-
{-# LANGUAGE ForeignFunctionInterface #-}

module Main where

import Foreign.Ptr
import Foreign.C.Types
import Foreign.C.String
import Foreign.Marshal.Alloc
import Foreign.Storable

data StatVF = StatVF {
      block_size :: Integer,
      free_blocks :: Integer
    } deriving (Eq, Show, Read)

#include <sys/statvfs.h>

instance Storable StatVF where
    sizeOf _ = #{size struct statvfs}
    alignment _ = alignment (undefined :: CInt)
    peek ptr = do
      bsize <- #{peek struct statvfs, f_bsize} ptr >>= return.fromCULong
      bfree <- #{peek struct statvfs, f_bfree} ptr >>= return.fromCULong
      return $ StatVF {block_size = bsize, free_blocks = bfree}
      where fromCULong :: CULong -> Integer
            fromCULong = toInteger
    poke _ _ = do
      return ()


foreign import ccall unsafe "sys/statvfs.h statvfs" c_statvfs :: CString -> Ptr StatVF -> IO CInt

statvfs :: String -> IO StatVF
statvfs path = 
    alloca $ \ptr -> 
        withCString path $ \s -> 
            do c_statvfs s (castPtr ptr)
               peek ptr

getFreeSpace s = statvfs s >>= \(StatVF a b) -> return (a*b)

